@@ -31,6 +31,8 @@
 #include <math.h>
 #include "corfile.h"
 
+#include "csdebug.h"
+
 #define SEGAMPS AMPLMSG
 #define SORMSG  RNGEMSG
 
@@ -170,18 +172,18 @@ int musmon(CSOUND *csound)
 
 #ifdef USE_DOUBLE
 #ifdef BETA
-    csound->Message(csound, Str("Csound version %s beta (double samples) %s\n"),
+    csound->Message(csound, Str("--Csound version %s beta (double samples) %s\n"),
                             CS_PACKAGE_VERSION, __DATE__);
 #else
-    csound->Message(csound, Str("Csound version %s (double samples) %s\n"),
+    csound->Message(csound, Str("--Csound version %s (double samples) %s\n"),
                             CS_PACKAGE_VERSION, __DATE__);
 #endif
 #else
 #ifdef BETA
-    csound->Message(csound, Str("Csound version %s beta (float samples) %s\n"),
+    csound->Message(csound, Str("--Csound version %s beta (float samples) %s\n"),
                             CS_PACKAGE_VERSION, __DATE__);
 #else
-    csound->Message(csound, Str("Csound version %s (float samples) %s\n"),
+    csound->Message(csound, Str("--Csound version %s (float samples) %s\n"),
                             CS_PACKAGE_VERSION, __DATE__);
 #endif
 #endif
@@ -199,9 +201,9 @@ int musmon(CSOUND *csound)
     dbfs_init(csound, csound->e0dbfs);
     csound->nspout = csound->ksmps * csound->nchnls;  /* alloc spin & spout */
     csound->nspin = csound->ksmps * csound->inchnls; /* JPff: in preparation */
-    csound->spin  = (MYFLT *) mcalloc(csound, csound->nspin * sizeof(MYFLT));
-    csound->spout = (MYFLT *) mcalloc(csound, csound->nspout * sizeof(MYFLT));
-    csound->auxspin  = (MYFLT *) mcalloc(csound, csound->nspin * sizeof(MYFLT));
+    csound->spin  = (MYFLT *) csound->Calloc(csound, csound->nspin * sizeof(MYFLT));
+    csound->spout = (MYFLT *) csound->Calloc(csound, csound->nspout * sizeof(MYFLT));
+    csound->auxspin = (MYFLT *) csound->Calloc(csound, csound->nspin *sizeof(MYFLT));
     /* memset(csound->maxamp, '\0', sizeof(MYFLT)*MAXCHNLS); */
     /* memset(csound->smaxamp, '\0', sizeof(MYFLT)*MAXCHNLS); */
     /* memset(csound->omaxamp, '\0', sizeof(MYFLT)*MAXCHNLS); */
@@ -294,7 +296,7 @@ int musmon(CSOUND *csound)
     //csound->scfp
     if (UNLIKELY(O->usingcscore)) {
       if (STA(lsect) == NULL) {
-        STA(lsect) = (EVENT*) mmalloc(csound, sizeof(EVENT));
+        STA(lsect) = (EVENT*) csound->Malloc(csound, sizeof(EVENT));
         STA(lsect)->op = 'l';
       }
       csound->Message(csound, Str("using Cscore processing\n"));
@@ -406,7 +408,7 @@ PUBLIC int csoundCleanup(CSOUND *csound)
     uint32_t n;
 
     if(csound->QueryGlobalVariable(csound,"::UDPCOM")
-         != NULL) UDPServerClose(csound);
+       != NULL) UDPServerClose(csound);
 
     while (csound->evtFuncChain != NULL) {
       p = (void*) csound->evtFuncChain;
@@ -427,7 +429,7 @@ PUBLIC int csoundCleanup(CSOUND *csound)
         csound->engineState.instrtxtp[0]->instance &&
         csound->engineState.instrtxtp[0]->instance->actflg)
       xturnoff_now(csound, csound->engineState.instrtxtp[0]->instance);
-      delete_pending_rt_events(csound);
+    delete_pending_rt_events(csound);
 
     if(csound->init_pass_loop == 1) {
       csoundLockMutex(csound->init_pass_threadlock);
@@ -893,6 +895,11 @@ int sensevents(CSOUND *csound)
     int     retval, sensType;
     int     conn, *sinp;
 
+    csdebug_data_t *data = (csdebug_data_t *) csound->csdebug_data;
+    if (data && data->status == CSDEBUG_STATUS_STOPPED) {
+        return 0; /* don't process events if we're in debug mode and stopped */
+    }
+
     if (UNLIKELY(csound->MTrkend && O->termifend)) {   /* end of MIDI file:  */
       deactivate_all_notes(csound);
       csound->Message(csound, Str("terminating.\n"));
@@ -1235,6 +1242,7 @@ int insert_score_event_at_sample(CSOUND *csound, EVTBLK *evt, int64_t time_ofs)
         /* calculate the length in beats */
         if (evt->p3orig > FL(0.0))
           evt->p3orig = (MYFLT) ((double) evt->p3orig / st->ibeatTime);
+        /* fall through required */
       case 'q':                         /* mute instrument */
         /* check for a valid instrument number or name */
         if (evt->strarg != NULL && ISSTRCOD(p[1]))
