@@ -32,6 +32,7 @@
 #include "namedins.h"
 #include "interlocks.h"
 #include "csound_orc_semantics.h"
+#include "csound_standard_types.h"
 
 #ifndef PARSER_DEBUG
 #define PARSER_DEBUG (0)
@@ -47,6 +48,8 @@ ORCTOKEN *add_token(CSOUND *csound, char *s, int type);
 //static ORCTOKEN *add_token_p(CSOUND *csound, char *s, int type, int val);
 extern int csound_orcget_lineno(void*);
 
+/* from csound_orc_compile.c */
+extern char** splitArgs(CSOUND* csound, char* argString);
 
 int get_opcode_type(OENTRY *ep)
 {
@@ -90,7 +93,7 @@ void init_symbtab(CSOUND *csound)
                 add_token(csound, shortName, get_opcode_type(ep));
 
                 if (shortName != ep->opname) {
-                    mfree(csound, shortName);
+                    csound->Free(csound, shortName);
                 }
             }
             items = items->next;
@@ -99,7 +102,7 @@ void init_symbtab(CSOUND *csound)
     }
 
 
-    mfree(csound, top);
+    csound->Free(csound, top);
 }
 
 ORCTOKEN *add_token(CSOUND *csound, char *s, int type)
@@ -119,7 +122,7 @@ ORCTOKEN *add_token(CSOUND *csound, char *s, int type)
       return a;
     }
     ans = new_token(csound, T_IDENT);
-    ans->lexeme = (char*)mmalloc(csound, 1+strlen(s));
+    ans->lexeme = (char*)csound->Malloc(csound, 1+strlen(s));
     strcpy(ans->lexeme, s);
     ans->type = type;
 
@@ -174,7 +177,7 @@ ORCTOKEN *lookup_token(CSOUND *csound, char *s, void *yyscanner)
     if (udoflag == 0) {
       if (isUDOAnsList(s)) {
         ans = new_token(csound, UDO_ANS_TOKEN);
-        ans->lexeme = (char*)mmalloc(csound, 1+strlen(s));
+        ans->lexeme = (char*)csound->Malloc(csound, 1+strlen(s));
         strcpy(ans->lexeme, s);
         return ans;
       }
@@ -184,7 +187,7 @@ ORCTOKEN *lookup_token(CSOUND *csound, char *s, void *yyscanner)
       if (csound->oparms->odebug) printf("Found UDO Arg List\n");
       if (isUDOArgList(s)) {
         ans = new_token(csound, UDO_ARGS_TOKEN);
-        ans->lexeme = (char*)mmalloc(csound, 1+strlen(s));
+        ans->lexeme = (char*)csound->Malloc(csound, 1+strlen(s));
         strcpy(ans->lexeme, s);
         return ans;
       }
@@ -193,16 +196,16 @@ ORCTOKEN *lookup_token(CSOUND *csound, char *s, void *yyscanner)
     a = cs_hash_table_get(csound, symbtab, s);
 
     if (a != NULL) {
-      ans = (ORCTOKEN*)mmalloc(csound, sizeof(ORCTOKEN));
+      ans = (ORCTOKEN*)csound->Malloc(csound, sizeof(ORCTOKEN));
       memcpy(ans, a, sizeof(ORCTOKEN));
       ans->next = NULL;
-      ans->lexeme = (char *)mmalloc(csound, strlen(a->lexeme) + 1);
+      ans->lexeme = (char *)csound->Malloc(csound, strlen(a->lexeme) + 1);
       strcpy(ans->lexeme, a->lexeme);
       return ans;
     }
 
     ans = new_token(csound, T_IDENT);
-    ans->lexeme = (char*)mmalloc(csound, 1+strlen(s));
+    ans->lexeme = (char*)csound->Malloc(csound, 1+strlen(s));
     strcpy(ans->lexeme, s);
 
     if (udoflag == -2 || namedInstrFlag == 1) {
@@ -215,12 +218,39 @@ ORCTOKEN *lookup_token(CSOUND *csound, char *s, void *yyscanner)
 }
 
 
+//static int is_optional_udo_in_arg(char* argtype) {
+//    return strchr("jOPVop", *argtype) != NULL;
+//}
+
+static char map_udo_in_arg_type(char in) {
+    if (strchr("ijop", in) != NULL) {
+        return 'i';
+    } else if (strchr("kKOPV", in) != NULL) {
+        return 'k';
+    }
+    return in;
+}
+
+static char map_udo_out_arg_type(char in) {
+    if (in == 'K') {
+        return 'k';
+    }
+    return in;
+}
+
+static void map_args(char* args) {
+    while (*args != '\0') {
+        *args = map_udo_out_arg_type(*args);
+        args++;
+    }
+}
+
 /**
  *
-  This function takes in the arguments from useropinfo in OENTRY and parses
-  them, filling the OENTRY input and output types and creating
-  the argument lists for xinset/xouset in insert.c
-  argument pointerlists, stored in useropinfo->in_ndx_list and
+  This function takes in the arguments from useropinfo in OENTRY and
+  parses them, filling the OENTRY input and output types and creating
+  the argument lists for xinset/xouset in insert.c argument
+  pointerlists, stored in useropinfo->in_ndx_list and
   useropinfo->out_ndx_list.
 
   The argument lists are terminated by a -1 and are set in the
@@ -244,224 +274,163 @@ ORCTOKEN *lookup_token(CSOUND *csound, char *s, void *yyscanner)
 static int parse_opcode_args(CSOUND *csound, OENTRY *opc)
 {
     OPCODINFO   *inm = (OPCODINFO*) opc->useropinfo;
-    char    *types, *otypes;
-    int     i, i_incnt, iv_incnt, iv_outcnt, a_incnt, k_incnt,
-            i_outcnt, a_outcnt, k_outcnt, err;
-    int     S_incnt, S_outcnt, f_outcnt, f_incnt, kv_incnt, kv_outcnt;
-    int16   *a_inlist, *k_inlist, *i_inlist, *a_outlist, *k_outlist, *i_outlist;
-    int16   *S_inlist, *S_outlist, *f_inlist, *f_outlist, *kv_inlist,
-            *kv_outlist, *iv_inlist, *iv_outlist;
+    char** in_args;
+    char** out_args;
+    char intypes[256];
+    char typeSpecifier[2];
+    char tempName[20];
+    int i = 0, err = 0;
+    ARRAY_VAR_INIT varInit;
 
-    /* count the number of arguments, and check types */
-    i = i_incnt = S_incnt = a_incnt = k_incnt = f_incnt = f_outcnt =
-        i_outcnt = S_outcnt = a_outcnt = k_outcnt = kv_incnt =
-        kv_outcnt = iv_outcnt = iv_incnt = err = 0;
-    types = inm->intypes; otypes = opc->intypes;
-    opc->dsblksiz = (uint16) sizeof(UOPCODE);
-    if (!strcmp(types, "0"))
-      types++;                  /* no input args */
-    while (*types) {
-      switch (*types) {
-      case 'a':
-        a_incnt++; *otypes++ = *types;
-        break;
-      case 'O':
-        k_incnt++; *otypes++ = 'O'; break;
-      case 'P':
-         k_incnt++;*otypes++ = 'P'; break;
-      case 'V':
-         k_incnt++;*otypes++ = 'V'; break;
-      case 'K':
-        i_incnt++;              /* also updated at i-time */
-      case 'k':
-        if(*(types+1) == '[') {
-          kv_incnt++;
-          *otypes++ = *types;
-          *otypes++ = *(types+1);
-          *otypes++ = *(types+2);
-          types+=2;
-          break;
+    typeSpecifier[1] = '\0';
+
+    // The following handles adding of extra 'o' type for optional
+    // ksmps arg for all UDO's
+    if (*inm->intypes == '0') {
+        intypes[0] = 'o';
+        intypes[1] = '\0';
+    } else {
+        snprintf(intypes, 256, "%so", inm->intypes);
+    }
+    in_args = splitArgs(csound, intypes);
+    out_args = splitArgs(csound, inm->outtypes);
+
+    if (*in_args[0] != '0') {
+      while (in_args[i] != NULL) {
+        char* in_arg = in_args[i];
+        snprintf(tempName, 20, "in%d", i);
+
+        if (*in_arg == '[') {
+          int dimensions = 0;
+          while (*in_arg == '[') {
+            dimensions += 1;
+            in_arg += 1;
+          }
+          typeSpecifier[0] = *in_arg;
+// printf("Dimensions: %d SubArgType: %s\n", dimensions, typeSpecifier);
+          CS_TYPE* type =
+            csoundGetTypeWithVarTypeName(csound->typePool, typeSpecifier);
+
+          if (type == NULL) {
+            synterr(csound, Str("invalid input type for opcode %s"), in_arg);
+            err++;
+            continue;
+          }
+
+          varInit.dimensions = dimensions;
+          varInit.type = type;
+          CS_VARIABLE* var = csoundCreateVariable(csound, csound->typePool,
+                                                  (CS_TYPE*)&CS_VAR_TYPE_ARRAY,
+                                                  tempName, &varInit);
+          var->dimensions = dimensions;
+          csoundAddVariable(csound, inm->in_arg_pool, var);
+        } else {
+          char c = map_udo_in_arg_type(*in_arg);
+          //                printf("found arg type %s -> %c\n", in_arg, c);
+
+          typeSpecifier[0] = c;
+          CS_TYPE* type =
+            csoundGetTypeWithVarTypeName(csound->typePool, typeSpecifier);
+
+          if (type == NULL) {
+            synterr(csound, Str("invalid input type for opcode %s"), in_arg);
+            err++;
+            continue;
+          }
+
+          CS_VARIABLE* var = csoundCreateVariable(csound, csound->typePool,
+                                                  type, tempName, NULL);
+          csoundAddVariable(csound, inm->in_arg_pool, var);
         }
-        k_incnt++; *otypes++ = 'k';
-        break;
-      case 'f':
-        f_incnt++; *otypes++ = *types;
-        break;
-      case 'i':
-      if(*(types+1) == '[') {
-          iv_incnt++;
-          *otypes++ = *types;
-          *otypes++ = *(types+1);
-          *otypes++ = *(types+2);
-          types+=2;
-          break;
-        }
-      case 'o':
-      case 'p':
-      case 'j':
-        i_incnt++; *otypes++ = *types;
-       break;
-      case 'S':
-        S_incnt++; *otypes++ = *types;
-        break;
-      default:
-        synterr(csound, Str("invalid input type for opcode %s"), inm->name);
-        err++; i--;
-      }
-      i++; types++;
-      if (UNLIKELY(i > OPCODENUMOUTS_MAX)) {
-        synterr(csound, Str("too many input args for opcode %s"), inm->name);
-        csound->LongJmp(csound, 1);
+        i++;
       }
     }
-    *otypes++ = 'o'; *otypes = '\0';    /* optional arg for local ksmps */
-    inm->inchns = i;                    /* total number of input chnls */
-    inm->perf_incnt = a_incnt + k_incnt + f_incnt + kv_incnt;
-    opc->dsblksiz += (uint16) (sizeof(MYFLT*) * i);
-    /* same for outputs */
+//    inm->inchns = i + 1; /* Add one for optional local ksmps */
+    inm->inchns = i - 1;
+
     i = 0;
-    types = inm->outtypes; otypes = opc->outypes;
-    if (!strcmp(types, "0"))
-      types++;                  /* no output args */
-    while (*types) {
-      if (UNLIKELY(i >= OPCODENUMOUTS_MAX)) {
-        synterr(csound, Str("too many output args for opcode %s"), inm->name);
-        csound->LongJmp(csound, 1);
-      }
-      switch (*types) {
-      case 'a':
-        a_outcnt++; *otypes++ = *types;
-        break;
-      case 'K':
-        i_outcnt++;             /* also updated at i-time */
-      case 'k':
-        if(*(types+1) == '[') {
-          kv_outcnt++;
-          *otypes++ = *types;
-          *otypes++ = *(types+1);
-          *otypes++ = *(types+2);
-          types+=2;
-          break;
-        }
-        k_outcnt++; *otypes++ = 'k';
-        break;
-      case 'f':
-        f_outcnt++; *otypes++ = *types;
-        break;
-      case 'i':
-         if(*(types+1) == '[') {
-          iv_outcnt++;
-          *otypes++ = *types;
-          *otypes++ = *(types+1);
-          *otypes++ = *(types+2);
-          types+=2;
-          break;
-        }
-        i_outcnt++; *otypes++ = *types;
-        break;
-      case 'S':
-        S_outcnt++; *otypes++ = *types;
-        break;
-      default:
-        synterr(csound, Str("invalid output type for opcode %s"), inm->name);
-        err++; i--;
-      }
-      i++; types++;
-    }
-    *otypes = '\0';
-    inm->outchns = i;                   /* total number of output chnls */
-    inm->perf_outcnt = a_outcnt + k_outcnt + f_outcnt + kv_outcnt;
+    if (*out_args[0] != '0') {
+      while(out_args[i] != NULL) {
+        char* out_arg = out_args[i];
+        snprintf(tempName, 20, "out%d", i);
 
-    opc->dsblksiz += (uint16) (sizeof(MYFLT*) * i);
+        if (*out_arg == '[') {
+          int dimensions = 0;
+          while (*out_arg == '[') {
+            dimensions += 1;
+            out_arg += 1;
+          }
+          typeSpecifier[0] = *out_arg;
+          //printf("Dimensions: %d SubArgType: %s\n", dimensions, typeSpecifier);
+          CS_TYPE* type =
+            csoundGetTypeWithVarTypeName(csound->typePool, typeSpecifier);
+
+          if (type == NULL) {
+            synterr(csound, Str("invalid output type for opcode %s"), out_arg);
+            err++;
+            continue;
+          }
+
+          varInit.dimensions = dimensions;
+          varInit.type = type;
+          CS_VARIABLE* var = csoundCreateVariable(csound, csound->typePool,
+                                                  (CS_TYPE*)&CS_VAR_TYPE_ARRAY,
+                                                  tempName, &varInit);
+          var->dimensions = dimensions;
+          csoundAddVariable(csound, inm->out_arg_pool, var);
+        } else {
+          char c = map_udo_out_arg_type(*out_arg);
+          //                printf("found arg type %s -> %c\n", out_arg, c);
+          typeSpecifier[0] = c;
+          CS_TYPE* type =
+            csoundGetTypeWithVarTypeName(csound->typePool, typeSpecifier);
+
+          if (type == NULL) {
+            synterr(csound, Str("invalid output type for opcode %s"), out_arg);
+            err++;
+            continue;
+          }
+
+          CS_VARIABLE* var = csoundCreateVariable(csound, csound->typePool, type,
+                                                  tempName, NULL);
+          csoundAddVariable(csound, inm->out_arg_pool, var);
+        }
+        i++;
+      }
+    }
+
+    inm->outchns = i;
+
+    opc->dsblksiz = (uint16) (sizeof(UOPCODE) +
+                              sizeof(MYFLT*) * (inm->inchns + inm->outchns));
     opc->dsblksiz = ((opc->dsblksiz + (uint16) 15)
                      & (~((uint16) 15)));   /* align (needed ?) */
-    /* now build index lists for the various types of arguments */
-    i = i_incnt + S_incnt + inm->perf_incnt + iv_incnt +
-        i_outcnt + S_outcnt + inm->perf_outcnt + iv_outcnt;
-    i_inlist = inm->in_ndx_list = (int16*) mmalloc(csound,
-                                                   sizeof(int16) * (i + 16));
-    S_inlist = i_inlist + i_incnt + 1;
-    iv_inlist =  S_inlist + S_incnt + 1;
-    a_inlist = iv_inlist + iv_incnt + 1;
-    k_inlist = a_inlist + a_incnt + 1;
-    f_inlist = k_inlist + k_incnt + 1;
-    kv_inlist = f_inlist + f_incnt + 1;
-    i = 0; types = inm->intypes;
-    while (*types) {
 
-      switch (*types++) {
 
-        case 'a': *a_inlist++ = i; break;
-        case 'O':
-        case 'P':
-        case 'V':
-        case 'k':
-        if(*(types) == '[') {
-          *kv_inlist++ = i;
-          types+=2;
-             break;
-          }
-         *k_inlist++ = i;
-        break;
-        case 'f': *f_inlist++ = i; break;
-        /* case '[': */
-        /*   if(*types=='i') *iv_inlist++ = i; */
-        /*   else *kv_inlist++ = i; */
-        /*   types+=2; */
-        /*   break; */
-        case 'K': *k_inlist++ = i;      /* also updated at i-time */
-        case 'i':
-         if(*(types) == '[') {
-          *iv_inlist++ = i;
-          types+=2;
-             break;
-          }
-        case 'o':
-        case 'p':
-        case 'j': *i_inlist++ = i; break;
-        case 'S': *S_inlist++ = i; break;
-      }
-      i++;
-    }
+    opc->intypes = cs_strdup(csound, intypes);
+    opc->outypes = cs_strdup(csound, (inm->outtypes[0] == '0') ? "" :
+                                                                 inm->outtypes);
 
-    /* put delimiters */
-    *i_inlist = *S_inlist = *iv_inlist = *a_inlist = *k_inlist =
-      *f_inlist = *kv_inlist = -1;
+    map_args(opc->intypes);
+    map_args(opc->outypes);
+//    /* count the number of arguments, and check types */
+//      default:
+//        synterr(csound, Str("invalid input type for opcode %s"), inm->name);
+//        err++; i--;
+//      }
+//      i++; types++;
+//      if (UNLIKELY(i > OPCODENUMOUTS_MAX)) {
+//        synterr(csound, Str("too many input args for opcode %s"), inm->name);
+//        csound->LongJmp(csound, 1);
+//      }
+//    }
+//      default:
+//        synterr(csound, Str("invalid output type for opcode %s"), inm->name);
+//        err++; i--;
+//      }
+//      i++; types++;
+//    }
 
-    i_outlist = inm->out_ndx_list = kv_inlist + 1;
-    S_outlist = i_outlist + i_outcnt + 1;
-    iv_outlist =  S_outlist + S_outcnt + 1;
-    a_outlist = iv_outlist + iv_outcnt + 1;
-    k_outlist = a_outlist + a_outcnt + 1;
-    f_outlist = k_outlist + k_outcnt + 1;
-    kv_outlist = f_outlist + f_outcnt + 1;
-    i = 0; types = inm->outtypes;
-    while (*types) {
-      switch (*types++) {
-        case 'a': *a_outlist++ = i; break;
-        case 'k':
-        if(*(types) == '[') {
-          *kv_outlist++ = i;
-          types+=2;
-             break;
-          }
-        *k_outlist++ = i; break;
-        case 'f': *f_outlist++ = i; break;
-        case 'K': *k_outlist++ = i;     /* also updated at i-time */
-        case 'i':
-        if(*(types) == '[') {
-          *iv_inlist++ = i;
-          types+=2;
-             break;
-        }
-        *i_outlist++ = i; break;
-        case 'S': *S_outlist++ = i; break;
-      }
-      i++;
-    }
-
-    *i_outlist = *S_outlist = *iv_outlist = *a_outlist = *k_outlist =
-      *f_outlist = *kv_outlist = -1;  /* put delimiters */
     return err;
 }
 
@@ -471,7 +440,7 @@ OENTRY* csound_find_internal_oentry(CSOUND* csound, OENTRY* oentry) {
     char *shortName;
     OENTRY *ep, *retVal = NULL;
 
-    if(oentry == NULL) {
+    if (oentry == NULL) {
         return NULL;
     }
     shortName = get_opcode_short_name(csound, oentry->opname);
@@ -493,7 +462,7 @@ OENTRY* csound_find_internal_oentry(CSOUND* csound, OENTRY* oentry) {
     }
 
     if (shortName != oentry->opname) {
-        mfree(csound, shortName);
+        csound->Free(csound, shortName);
     }
 
     return retVal;
@@ -540,11 +509,13 @@ int add_udo_definition(CSOUND *csound, char *opname,
     }
 
     /* IV - Oct 31 2002 */
-    /* store the name in a linked list (note: must use mcalloc) */
-    inm = (OPCODINFO *) mcalloc(csound, sizeof(OPCODINFO));
+    /* store the name in a linked list (note: must use csound->Calloc) */
+    inm = (OPCODINFO *) csound->Calloc(csound, sizeof(OPCODINFO));
     inm->name = cs_strdup(csound, opname);
     inm->intypes = intypes;
     inm->outtypes = outtypes;
+    inm->in_arg_pool = csoundCreateVarPool(csound);
+    inm->out_arg_pool = csoundCreateVarPool(csound); 
 
     inm->prv = csound->opcodeInfo;
     csound->opcodeInfo = inm;
@@ -561,7 +532,7 @@ int add_udo_definition(CSOUND *csound, char *opname,
 
     /* check in/out types and copy to the opcode's */
     /* IV - Sep 8 2002: opcodes have an optional arg for ksmps */
-    newopc->outypes = mmalloc(csound, strlen(outtypes) + 1
+    newopc->outypes = csound->Malloc(csound, strlen(outtypes) + 1
                                       + strlen(intypes) + 2);
     newopc->intypes = &(newopc->outypes[strlen(outtypes) + 1]);
 
