@@ -558,7 +558,8 @@ void sfopenin(CSOUND *csound)           /* init for continuous soundin */
  inset:
     /* calc inbufsize reqd */
     STA(inbufsiz) = (unsigned) (O->inbufsamps * sizeof(MYFLT));
-    STA(inbuf) = (MYFLT*) mcalloc(csound, STA(inbufsiz)); /* alloc inbuf space */
+    STA(inbuf) = (MYFLT*) csound->Calloc(csound,
+                                         STA(inbufsiz)); /* alloc inbuf space */
     if (STA(pipdevout) == 2)
       csound->Message(csound,
                       Str("reading %d sample blks of %d-bit floats from %s \n"),
@@ -730,6 +731,8 @@ void sfopenout(CSOUND *csound)                  /* init for sound out       */
     /* open file */
     if (STA(pipdevout)) {
       STA(outfile) = sf_open_fd(osfd, SFM_WRITE, &sfinfo, 0);
+      sf_command(STA(outfile), SFC_SET_VBR_ENCODING_QUALITY,
+                 &O->quality, sizeof(double));
 #ifdef PIPES
       if (STA(outfile) == NULL) {
         char fmt_name[6];
@@ -751,6 +754,8 @@ void sfopenout(CSOUND *csound)                  /* init for sound out       */
                                     "for use in pipe\n"), fmt_name);
         sfinfo.format = TYPE2SF(O->filetyp) | FORMAT2SF(O->outformat);
         STA(outfile) = sf_open_fd(osfd, SFM_WRITE, &sfinfo, 0);
+        sf_command(STA(outfile), SFC_SET_VBR_ENCODING_QUALITY,
+                 &O->quality, sizeof(double));
       }
 #endif
       if (UNLIKELY(STA(outfile) == NULL))
@@ -763,6 +768,8 @@ void sfopenout(CSOUND *csound)                  /* init for sound out       */
         csoundDie(csound, Str("sfinit: cannot open %s"), fName);
       STA(sfoutname) = fullName;
       STA(outfile)   = sf_open(fullName, SFM_WRITE, &sfinfo);
+      sf_command(STA(outfile), SFC_SET_VBR_ENCODING_QUALITY,
+                 &O->quality, sizeof(double));
       if (UNLIKELY(STA(outfile) == NULL))
         csoundDie(csound, Str("sfinit: cannot open %s"), fullName);
       /* only notify the host if we opened a real file, not stdout or a pipe */
@@ -824,7 +831,7 @@ void sfopenout(CSOUND *csound)                  /* init for sound out       */
     O->sfsampsize = (int) sfsampsize(FORMAT2SF(O->outformat));
     /* calc outbuf size & alloc bufspace */
     STA(outbufsiz) = O->outbufsamps * sizeof(MYFLT);
-    STA(outbufp)   = STA(outbuf) = mmalloc(csound, STA(outbufsiz));
+    STA(outbufp)   = STA(outbuf) = csound->Malloc(csound, STA(outbufsiz));
     if (STA(pipdevout) == 2)
       csound->Message(csound,
                       Str("writing %d sample blks of %d-bit floats to %s \n"),
@@ -1009,9 +1016,9 @@ void iotranset(CSOUND *csound)
       return;
     }
     STA(inbufsiz)  = (unsigned int) (O->inbufsamps * (int) sizeof(MYFLT));
-    STA(inbuf)     = (MYFLT*) mcalloc(csound, STA(inbufsiz));
+    STA(inbuf)     = (MYFLT*) csound->Calloc(csound, STA(inbufsiz));
     STA(outbufsiz) = (unsigned int) (O->outbufsamps * (int) sizeof(MYFLT));
-    STA(outbuf)    = (MYFLT*) mcalloc(csound, STA(outbufsiz));
+    STA(outbuf)    = (MYFLT*) csound->Calloc(csound, STA(outbufsiz));
     STA(outbufp)   = STA(outbuf);
     O->sfread      = 1;
     O->sfwrite     = 1;
