@@ -28,7 +28,6 @@
 #include "csdl.h"                               /*      PMIDI.C         */
 #include "csGblMtx.h"
 #include "midiops.h"
-#include "oload.h"
 #include <portmidi.h>
 #include <porttime.h>
 
@@ -123,29 +122,6 @@ static PmDeviceInfo *portMidi_getDeviceInfo(int dev, int output)
       return NULL;
     return ((PmDeviceInfo*)Pm_GetDeviceInfo((PmDeviceID) i));
 }
-
-static void portMidi_listDevices(CSOUND *csound, int output)
-{
-    int           i, cnt;
-    PmDeviceInfo  *info;
-
-    cnt = portMidi_getDeviceCount(output);
-    if (UNLIKELY(cnt < 1))
-      return;
-    if (output)
-      csound->Message(csound, Str("The available MIDI out devices are:\n"));
-    else
-      csound->Message(csound, Str("The available MIDI in devices are:\n"));
-    for (i = 0; i < cnt; i++) {
-      info = portMidi_getDeviceInfo(i, output);
-      if (info->interf != NULL)
-        csound->Message(csound, " %3d: %s (%s)\n", i, info->name, info->interf);
-      else
-        csound->Message(csound, " %3d: %s\n", i, info->name);
-
-    }
-}
-
 /* reference count for PortMidi initialisation */
 
 static unsigned long portmidi_init_cnt = 0UL;
@@ -184,9 +160,48 @@ static int start_portmidi(CSOUND *csound)
       csound->ErrorMsg(csound, Str(errMsg));
       return -1;
     }
+    csound_global_mutex_unlock();
     return csound->RegisterResetCallback(csound, NULL, stop_portmidi);
 }
 
+static int listDevices(CSOUND *csound, CS_MIDIDEVICE *list, int isOutput){
+  int i, cnt;
+  PmDeviceInfo  *info;
+  char tmp[64];
+  char *drv = (char*) (csound->QueryGlobalVariable(csound, "_RTMIDI"));
+
+  if (UNLIKELY(start_portmidi(csound) != 0))
+      return 0;
+
+  cnt = portMidi_getDeviceCount(isOutput);
+  if(list == NULL) return cnt;
+  for (i = 0; i < cnt; i++) {
+      info = portMidi_getDeviceInfo(i, isOutput);
+      strncpy(list[i].device_name, info->name, 63);
+      snprintf(tmp, 64, "%d", i);
+      strncpy(list[i].device_id, tmp, 63);
+      list[i].isOutput = isOutput;
+      if (info->interf != NULL)
+         strncpy(list[i].interface_name, info->interf, 63);
+      else strcpy(list[i].interface_name, "");
+     strncpy(list[i].midi_module, drv, 63);
+  }
+  return cnt;
+}
+
+static void portMidi_listDevices(CSOUND *csound, int output)
+{
+    int i,n = listDevices(csound, NULL, output);
+    CS_MIDIDEVICE *devs =
+      (CS_MIDIDEVICE *) csound->Malloc(csound, n*sizeof(CS_MIDIDEVICE));
+    listDevices(csound, devs, output);
+    for(i=0; i < n; i++)
+      csound->Message(csound, "%s: %s (%s)\n",
+                      devs[i].device_id, devs[i].device_name, devs[i].midi_module);
+    csound->Free(csound, devs);
+}
+
+
 static int OpenMidiInDevice_(CSOUND *csound, void **userData, const char *dev)
 {
     int         cntdev, devnum, opendevs, i;
@@ -225,7 +240,7 @@ static int OpenMidiInDevice_(CSOUND *csound, void **userData, const char *dev)
       devnum = -1;
     }
 
-    if (UNLIKELY(cntdev < 1 && dev[0] != 'a')) {
+    if (UNLIKELY(cntdev < 1 && (dev==NULL || dev[0] != 'a'))) {
       return portMidiErrMsg(csound, Str("no input devices are available"));
     }
     opendevs = 0;
@@ -256,6 +271,11 @@ static int OpenMidiInDevice_(CSOUND *csound, void **userData, const char *dev)
                  (PmDeviceID) portMidi_getRealDeviceID(i, 0),
                          NULL, 512L, (PmTimeProcPtr) NULL, NULL);
         if (UNLIKELY(retval != pmNoError)) {
+          // Prevent leaking memory from "data"
+          if (data) {
+            next = data->next;
+            free(data);
+          }
           return portMidiErrMsg(csound, Str("error opening input device %d: %s"),
                                           i, Pm_GetErrorText(retval));
         }
@@ -321,6 +341,7 @@ static int OpenMidiOutDevice_(CSOUND *csound, void **userData, const char *dev)
     }
     *userData = (void*) midistream;
     /* report success */
+
     return 0;
 }
 
@@ -400,6 +421,9 @@ static int WriteMidiData_(CSOUND *csound, void *userData,
       return 0;
     n = 0;
     do {
+      int time = csound->GetCurrentTimeSamples(csound)/csound->GetSr(csound);
+      //printf("jitter: %d \n",
+      //       Pt_Time(NULL) - (int)(1000*time/csound->GetSr(csound)));
       st = (int)*(mbuf++);
       if (UNLIKELY(st < 0x80)) {
         portMidiErrMsg(csound, Str("invalid MIDI out data"));
@@ -425,8 +449,9 @@ static int WriteMidiData_(CSOUND *csound, void *userData,
         mev.message |= (PmMessage) Pm_Message(0, 0, (int)*(mbuf++));
       if (UNLIKELY(Pm_Write(midistream, &mev, 1L) != pmNoError))
         portMidiErrMsg(csound, Str("MIDI out: error writing message"));
-      else
+      else {
         n += (datbyts[(st - 0x80) >> 4] + 1);
+      }
     } while (nbytes > 0);
     /* return the number of bytes written */
     return n;
@@ -467,14 +492,14 @@ static int CloseMidiOutDevice_(CSOUND *csound, void *userData)
 PUBLIC int csoundModuleCreate(CSOUND *csound)
 {
     /* nothing to do, report success */
-    csound->Message(csound, Str("PortMIDI real time MIDI plugin for Csound\n"));
+    // csound->Message(csound, Str("PortMIDI real time MIDI plugin for Csound\n"));
     return 0;
 }
 
 PUBLIC int csoundModuleInit(CSOUND *csound)
 {
     char    *drv;
-
+    csound->module_list_add(csound, "portmidi", "midi");
     drv = (char*) (csound->QueryGlobalVariable(csound, "_RTMIDI"));
     if (drv == NULL)
       return 0;
@@ -488,6 +513,13 @@ PUBLIC int csoundModuleInit(CSOUND *csound)
     csound->SetExternalMidiOutOpenCallback(csound, OpenMidiOutDevice_);
     csound->SetExternalMidiWriteCallback(csound, WriteMidiData_);
     csound->SetExternalMidiOutCloseCallback(csound, CloseMidiOutDevice_);
+    csound->SetMIDIDeviceListCallback(csound,listDevices);
+
+    return 0;
+}
+
+PUBLIC int csoundModuleDestroy(CSOUND *csound) {
+
     return 0;
 }
 
