@@ -901,11 +901,11 @@ static inline void tabensure(CSOUND *csound, ARRAYDAT *p, int size)
     if (p->data==NULL || p->dimensions == 0 ||
         (p->dimensions==1 && p->sizes[0] < size)) {
       uint32_t ss = sizeof(MYFLT)*size;
-      if (p->data==NULL) p->data = (MYFLT*)mmalloc(csound, ss);
-      else p->data = (MYFLT*) mrealloc(csound, p->data, ss);
+      if (p->data==NULL) p->data = (MYFLT*)csound->Malloc(csound, ss);
+      else p->data = (MYFLT*) csound->ReAlloc(csound, p->data, ss);
       p->dimensions = 1;
       p->arrayMemberSize = sizeof(MYFLT);
-      p->sizes = (int*)mmalloc(csound, sizeof(int));
+      p->sizes = (int*)csound->Malloc(csound, sizeof(int));
       p->sizes[0] = size;
     }
 }
@@ -1191,7 +1191,7 @@ static int i_infile_(CSOUND *csound, I_INFILE *p, int istring)
             goto newcycle;
           }
           while (isdigit(*cfp) || *cfp == '.' || *cfp == '+' || *cfp == '-') {
-            *(++cfp) = cc = getc(fp);
+            *(++cfp) = (char)(cc = getc(fp));
           }
           *++cfp = '\0';        /* Must terminate string */
           *(args[j]) = (MYFLT) atof(cf);
@@ -1226,7 +1226,7 @@ static int i_infile_(CSOUND *csound, I_INFILE *p, int istring)
       }
       break;
     case 2: /* binary floats without loop */
-      fseek(fp, p->currpos * sizeof(float) * nargs, SEEK_SET);
+      if (fseek(fp, p->currpos * sizeof(float) * nargs, SEEK_SET)<0) return NOTOK;
       p->currpos++;
       for (j = 0; j < nargs; j++) {
         if (fread(args[j], sizeof(float), 1, fp));
@@ -1418,7 +1418,7 @@ static void sprints(char *outstring, char *fmt, MYFLT **kvals, int32 numVals)
 
           switch (*segwaiting) {
           case '%':
-            sprintf(outstring, "%%");
+            snprintf(outstring, 8192, "%%");
             j--;
             break;
           case 'd':
@@ -1428,17 +1428,17 @@ static void sprints(char *outstring, char *fmt, MYFLT **kvals, int32 numVals)
           case 'X':
           case 'u':
           case 'c':
-            sprintf(outstring, strseg, (int) MYFLT2LRND(*kvals[j]));
+            snprintf(outstring, 8192, strseg, (int) MYFLT2LRND(*kvals[j]));
             break;
           case 'h':
-            sprintf(outstring, strseg, (int16) MYFLT2LRND(*kvals[j]));
+            snprintf(outstring, 8192, strseg, (int16) MYFLT2LRND(*kvals[j]));
             break;
           case 'l':
-            sprintf(outstring, strseg, (int32) MYFLT2LRND(*kvals[j]));
+            snprintf(outstring, 8192, strseg, (int32) MYFLT2LRND(*kvals[j]));
             break;
 
           default:
-            sprintf(outstring, strseg, *kvals[j]);
+            snprintf(outstring, 8192, strseg, *kvals[j]);
             break;
           }
           outstring += strlen(outstring);
@@ -1469,7 +1469,7 @@ static void sprints(char *outstring, char *fmt, MYFLT **kvals, int32 numVals)
       if (segwaiting) {
         switch (*segwaiting) {
           case '%':
-            sprintf(outstring, "%%");
+            strncpy(outstring, "%%", 8192);
             j--;
             break;
         case 'd':
@@ -1479,22 +1479,22 @@ static void sprints(char *outstring, char *fmt, MYFLT **kvals, int32 numVals)
         case 'X':
         case 'u':
         case 'c':
-          sprintf(outstring, strseg, (int) MYFLT2LRND(*kvals[j]));
+          snprintf(outstring, 8192, strseg, (int) MYFLT2LRND(*kvals[j]));
           break;
         case 'h':
-          sprintf(outstring, strseg, (int16) MYFLT2LRND(*kvals[j]));
+          snprintf(outstring, 8192, strseg, (int16) MYFLT2LRND(*kvals[j]));
           break;
         case 'l':
-          sprintf(outstring, strseg, (long) MYFLT2LRND(*kvals[j]));
+          snprintf(outstring, 8192, strseg, (long) MYFLT2LRND(*kvals[j]));
           break;
 
         default:
-          sprintf(outstring, strseg, *kvals[j]);
+          snprintf(outstring, 8192, strseg, *kvals[j]);
           break;
         }
       }
       else
-        sprintf(outstring, strseg);
+        snprintf(outstring, 8192, "%s", strseg);
     }
 }
 
@@ -1504,7 +1504,7 @@ static int fprintf_k(CSOUND *csound, FPRINTF *p)
 
     (void) csound;
     sprints(string, p->txtstring, p->argums, p->INOCOUNT - 2);
-    fprintf(p->f.f, string);
+    fprintf(p->f.f, "%s", string);
 
     return OK;
 }
@@ -1517,7 +1517,7 @@ static int fprintf_i(CSOUND *csound, FPRINTF *p)
     if (UNLIKELY(fprintf_set(csound, p) != OK))
       return NOTOK;
     sprints(string, p->txtstring, p->argums, p->INOCOUNT - 2);
-    fprintf(p->f.f, string);
+    fprintf(p->f.f,"%s", string);
     /* fflush(p->f.f); */
     return OK;
 }
@@ -1529,7 +1529,7 @@ static int fprintf_i_S(CSOUND *csound, FPRINTF *p)
     if (UNLIKELY(fprintf_set_S(csound, p) != OK))
       return NOTOK;
     sprints(string, p->txtstring, p->argums, p->INOCOUNT - 2);
-    fprintf(p->f.f, string);
+    fprintf(p->f.f, "%s", string);
     /* fflush(p->f.f); */
     return OK;
 }
