@@ -35,15 +35,20 @@ typedef struct SNDLOAD_OPCODE_ {
     MYFLT   *iLoopMode1, *iLoopStart1, *iLoopEnd1;
 } SNDLOAD_OPCODE;
 
-static int sndload_opcode_init(CSOUND *csound, SNDLOAD_OPCODE *p)
+static int sndload_opcode_init_(CSOUND *csound, SNDLOAD_OPCODE *p, int isstring)
 {
     char        *fname;
     SNDMEMFILE  *sf;
     SF_INFO     sfinfo;
     int         sampleFormat, loopMode;
 
-    fname = csound->strarg2name(csound, (char*) NULL, p->Sfname, "soundin.",
-                                (int) csound->GetInputArgSMask(p));
+    if(isstring) fname = ((STRINGDAT *)p->Sfname)->data;
+    else {
+      if(ISSTRCOD(*p->Sfname))
+        fname = csound->Strdup(csound, get_arg_string(csound, *p->Sfname));
+      else
+        fname = csound->strarg2name(csound, (char*) NULL, p->Sfname, "soundin.", 0);
+    }
     memset(&sfinfo, 0, sizeof(SF_INFO));
     sampleFormat = (int) MYFLT2LRND(*(p->iFormat));
     sfinfo.format = (int) TYPE2SF(TYP_RAW);
@@ -69,7 +74,7 @@ static int sndload_opcode_init(CSOUND *csound, SNDLOAD_OPCODE *p)
       tmp = (int) MYFLT2LRND(*(p->iChannels));
       sfinfo.channels = (tmp > 0 ? tmp : 1);
       tmp = (int) MYFLT2LRND(*(p->iSampleRate));
-      sfinfo.samplerate = (tmp > 0 ? tmp : (int) MYFLT2LRND(csound->esr));
+      sfinfo.samplerate = (tmp > 0 ? tmp : (int) MYFLT2LRND(CS_ESR));
     }
     sf = csound->LoadSoundFile(csound, fname,  &sfinfo);
     if (UNLIKELY(sf == NULL)) {
@@ -106,6 +111,15 @@ static int sndload_opcode_init(CSOUND *csound, SNDLOAD_OPCODE *p)
     return OK;
 }
 
+static int sndload_opcode_init(CSOUND *csound, SNDLOAD_OPCODE *p){
+  return sndload_opcode_init_(csound,p,0);
+}
+
+static int sndload_opcode_init_S(CSOUND *csound, SNDLOAD_OPCODE *p){
+  return sndload_opcode_init_(csound,p,0);
+}
+
+
  /* ------------------------------------------------------------------------ */
 
 #define LOSCILX_MAXOUTS         (16)
@@ -169,13 +183,16 @@ static int loscilx_opcode_init(CSOUND *csound, LOSCILX_OPCODE *p)
     p->dataPtr = NULL;
     nChannels = csound->GetOutputArgCnt(p);
     if (UNLIKELY(nChannels < 1 || nChannels > LOSCILX_MAXOUTS))
-      csound->Die(csound, Str("loscilx: invalid number of output arguments"));
+      return csound->InitError(csound,
+                               Str("loscilx: invalid number of output arguments"));
     p->nChannels = nChannels;
-    if (csound->GetInputArgSMask(p) != 0UL) {
+    if (ISSTRCOD(*p->ifn)) {
       SNDMEMFILE  *sf;
 
       p->usingFtable = 0;
-      sf = csound->LoadSoundFile(csound, (char*) p->ifn, (SF_INFO *) NULL);
+      sf = csound->LoadSoundFile(csound,
+                                 (char*) get_arg_string(csound, *p->ifn),
+                                 (SF_INFO *) NULL);
       if (UNLIKELY(sf == NULL))
         return csound->InitError(csound, Str("could not load '%s'"),
                                          (char*) p->ifn);
@@ -203,10 +220,10 @@ static int loscilx_opcode_init(CSOUND *csound, LOSCILX_OPCODE *p)
       }
       if (*(p->ibas) > FL(0.0)) {
         frqScale = sf->sampleRate
-                   / ((double) csound->esr * (double) *(p->ibas));
+                   / ((double) CS_ESR * (double) *(p->ibas));
       }
       else
-        frqScale = sf->sampleRate / ((double) csound->esr * sf->baseFreq);
+        frqScale = sf->sampleRate / ((double) CS_ESR * sf->baseFreq);
       p->ampScale = (MYFLT) sf->scaleFac * csound->e0dbfs;
       p->nFrames = (int32) sf->nFrames;
     }
@@ -238,7 +255,7 @@ static int loscilx_opcode_init(CSOUND *csound, LOSCILX_OPCODE *p)
       if (*(p->ibas) > FL(0.0)) {
         if (ftp->gen01args.sample_rate > FL(0.0))
           frqScale = (double) ftp->gen01args.sample_rate
-                     / ((double) csound->esr * (double) *(p->ibas));
+                     / ((double) CS_ESR * (double) *(p->ibas));
         else
           frqScale = 1.0 / (double) *(p->ibas);
       }
@@ -246,7 +263,7 @@ static int loscilx_opcode_init(CSOUND *csound, LOSCILX_OPCODE *p)
         frqScale = (double) ftp->cpscvt * (1.0 / (double) LOFACT);
       }
       else if (ftp->gen01args.sample_rate > FL(0.0))
-        frqScale = (double) ftp->gen01args.sample_rate / (double) csound->esr;
+        frqScale = (double) ftp->gen01args.sample_rate / (double) CS_ESR;
       p->ampScale = FL(1.0);
       p->nFrames = ftp->flenfrms + 1L;
     }
@@ -344,7 +361,10 @@ static inline void init_sine_gen(double a, double f, double p, double c,
 
 static int loscilx_opcode_perf(CSOUND *csound, LOSCILX_OPCODE *p)
 {
-    int     i, j;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t i, nsmps = CS_KSMPS;
+    int     j;
     double  frac_d, pidwarp_d = 0.0, c = 0.0;
     MYFLT   frac, ampScale, winFact = p->winFact;
     int32   ndx;
@@ -393,9 +413,12 @@ static int loscilx_opcode_perf(CSOUND *csound, LOSCILX_OPCODE *p)
       winFact = (MYFLT) (((double) p->winFact - tmp1) * tmp2 + tmp1);
     }
     ampScale = *(p->xamp) * p->ampScale;
-
-    i = 0;
-    do {
+    if (UNLIKELY(offset)) memset(p->ar, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&p->ar[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (i = offset; i<nsmps; i++) {
 
       frac_d = loscilx_phase_frac(p->curPos);
       frac = (MYFLT) frac_d;
@@ -660,20 +683,23 @@ static int loscilx_opcode_perf(CSOUND *csound, LOSCILX_OPCODE *p)
       else
         p->curPos += p->curPosInc;
 
-    } while (++i < csound->ksmps);
+    }
 
     return OK;
  err1:
-    return csound->PerfError(csound, Str("loscilx: not initialised"));
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("loscilx: not initialised"));
 }
 
  /* ------------------------------------------------------------------------ */
 
 static OENTRY loscilx_localops[] = {
-  { "sndload",  sizeof(SNDLOAD_OPCODE), 1,  "",                 "Tooooojjoo",
+  { "sndload",  sizeof(SNDLOAD_OPCODE), 0, 1,  "",                 "iooooojjoo",
     (SUBR) sndload_opcode_init, (SUBR) NULL, (SUBR) NULL                      },
-  { "loscilx",  sizeof(LOSCILX_OPCODE), TR|5,  "mmmmmmmmmmmmmmmm", "xkToojjoo",
+ { "sndload.S",  sizeof(SNDLOAD_OPCODE), 0, 1,  "",                 "Sooooojjoo",
+    (SUBR) sndload_opcode_init_S, (SUBR) NULL, (SUBR) NULL                      },
+  { "loscilx",  sizeof(LOSCILX_OPCODE), TR, 5,  "mmmmmmmmmmmmmmmm", "xkioojjoo",
     (SUBR) loscilx_opcode_init, (SUBR) NULL, (SUBR) loscilx_opcode_perf       }
 };
 
-LINKAGE1(loscilx_localops)
+LINKAGE_BUILTIN(loscilx_localops)
