@@ -1707,7 +1707,7 @@ int trnset(CSOUND *csound, TRANSEG *p)
         segp->cnt = 0;
       else
         segp->cnt = (int32)(dur * CS_EKR);
-        segp->nxtpt = nxtval;
+      segp->nxtpt = nxtval;
       segp->val = val;
       if (alpha == FL(0.0)) {
         segp->c1 = (nxtval-val)/d;
@@ -1761,7 +1761,7 @@ int trnset_bkpt(CSOUND *csound, TRANSEG *p)
         segp->cnt = 0;
       else
         segp->cnt = (int32)(dur * CS_EKR);
-        segp->nxtpt = nxtval;
+      segp->nxtpt = nxtval;
       segp->val = val;
       if (alpha == FL(0.0)) {
         segp->c1 = (nxtval-val)/d;
@@ -1907,7 +1907,7 @@ int trnsetr(CSOUND *csound, TRANSEG *p)
         segp->cnt = 0;
       else
         segp->cnt = (int32)(dur * CS_EKR);
-        segp->nxtpt = nxtval;
+      segp->nxtpt = nxtval;
       segp->val = val;
       if (alpha == FL(0.0)) {
         segp->c1 = (nxtval-val)/d;
@@ -2154,7 +2154,7 @@ int lpf18db(CSOUND *csound, LPF18 *p)
     MYFLT lastin = p->lastin;
     double value = 0.0;
     int   flag = 1;
-    MYFLT lfc, lrs, kres, kfcn, kp, kp1,  kp1h;
+    MYFLT lfc=0, lrs=0, kres=0, kfcn=0, kp=0, kp1=0,  kp1h=0;
     double lds = 0.0;
 
     if (UNLIKELY(offset)) memset(ar, '\0', offset*sizeof(MYFLT));
