@@ -19,6 +19,7 @@
  */
 
 #include <Python.h>
+#include <sysdep.h>
 #include "csdl.h"
 #include "pythonopcodes.h"
 #include "pythonhelper.h"
@@ -49,14 +50,17 @@ static void format_call_statement2(char *statement, char *callable,
 
     statement[0] = '\0';
     if (argc-skip > 0) {
-      sprintf(statement, "%s(%0.6f", callable, *(argv[skip]));
+      snprintf(statement, 1024, "%s(%0.6f", callable, *(argv[skip]));
       for (i = skip+1; i < argc; ++i) {
-        sprintf(statement + strlen(statement), ", %f", *(argv[i]));
+        snprintf(statement + strlen(statement), 1024 - strlen(statement),
+                ", %f", *(argv[i]));
       }
-      strcat(statement, ")");
+      // MKG 2014 Jan 29: No linkage for strlcat in py.dll on MinGW.
+      //strlcat(statement, ")", 1024);
+      strncat(statement, ")", 1023 - strlen(statement)); statement[1023] = '\0';
     }
     else {
-      sprintf(statement, "%s()", callable);
+      snprintf(statement, 1024, "%s()", callable);
     }
 
 }
@@ -68,14 +72,17 @@ static void format_call_statement(char *statement, char *callable,
 
     statement[0] = '\0';
     if (argc > 0) {
-      sprintf(statement, "%s(%0.6f", callable, *(argv[0]));
+      snprintf(statement, 1024, "%s(%0.6f", callable, *(argv[0]));
       for (i = 1; i < argc - skip; ++i) {
-        sprintf(statement + strlen(statement), ", %f", *(argv[i]));
+        snprintf(statement + strlen(statement), 1024-strlen(statement),
+                 ", %f", *(argv[i]));
       }
-      strcat(statement, ")");
+      // MKG 2014 Jan 29: No linkage for strlcat in py.dll on MinGW.
+      //strlcat(statement, ")", 1024);
+      strncat(statement, ")", 1023-strlen(statement)); statement[1023] = '\0';
     }
     else {
-      sprintf(statement, "%s()", callable);
+      snprintf(statement, 1024, "%s()", callable);
     }
 }
 
