@@ -120,8 +120,9 @@ static void SoundFontLoad(CSOUND *csound, char *fname)
       csound->ErrorMsg(csound, Str("Sfload: cannot use globals"));
       return;
     }
-    strcpy(soundFont->name, csound->GetFileName(fd));
-    chunk_read(fil, &soundFont->chunk.main_chunk);
+    strncpy(soundFont->name, csound->GetFileName(fd), 256);
+    if (UNLIKELY(chunk_read(fil, &soundFont->chunk.main_chunk)<0))
+      csound->Message(csound, Str("sfont: failed to read file\n"));
     csound->FileClose(csound, fd);
     globals->soundFont = soundFont;
     fill_SfPointers(csound);
@@ -166,7 +167,7 @@ static int SfLoad_(CSOUND *csound, SFLOAD *p, int istring)
     *p->ihandle = (float) globals->currSFndx;
     sf = &globals->sfArray[globals->currSFndx];
     qsort(sf->preset, sf->presets_num, sizeof(presetType),
-          (int (*)(const void *, const void * )) compare);
+        (int (*)(const void *, const void * )) compare);
     csound->Free(csound,fname);
     if (UNLIKELY(++globals->currSFndx>=globals->maxSFndx)) {
       globals->maxSFndx += 5;
@@ -1519,7 +1520,7 @@ static void fill_SfStruct(CSOUND *csound)
 
     size = phdrChunk->ckSize / sizeof(sfPresetHeader);
     soundFont->presets_num = size;
-    preset = (presetType *) malloc(size * sizeof(sfPresetHeader));
+    preset = (presetType *) malloc(size * sizeof(presetType));
     for (j=0; j < size; j++) {
       preset[j].name = phdr[j].achPresetName;
       if (strcmp(preset[j].name,"EOP")==0) {
@@ -1648,12 +1649,14 @@ static void fill_SfStruct(CSOUND *csound)
                         split->num= num;
                         split->sample = &shdr[num];
                         if (UNLIKELY(split->sample->sfSampleType & 0x8000)) {
-                            csound->ErrorMsg(csound, Str("SoundFont file \"%s\" "
-                                                  "contains ROM samples !\n"
-                                                  "At present time only RAM "
-                                                  "samples are allowed "
-                                                  "by sfload.\n"
-                                                  "Session aborted !"), Gfname);
+                          free(preset);
+                          csound->ErrorMsg(csound, Str("SoundFont file \"%s\" "
+                                                       "contains ROM samples !\n"
+                                                       "At present time only RAM "
+                                                       "samples are allowed "
+                                                       "by sfload.\n"
+                                                       "Session aborted !"),
+                                           Gfname);
                             return;
                         }
                         sglobal_zone = 0;
@@ -1886,11 +1889,12 @@ static void fill_SfStruct(CSOUND *csound)
                   split->num= num;
                   split->sample = &shdr[num];
                   if (UNLIKELY(split->sample->sfSampleType & 0x8000)) {
+                    free(instru);
                     csound->ErrorMsg(csound, Str("SoundFont file \"%s\" contains "
                                             "ROM samples !\n"
                                             "At present time only RAM samples "
                                             "are allowed by sfload.\n"
-                                            "Session aborted !"), Gfname);
+                                                 "Session aborted !"), Gfname);
                     return;
                   }
                   sglobal_zone = 0;
