@@ -532,15 +532,15 @@ int spdspset(CSOUND *csound, SPECDISP *p)
       SPECDAT *specp = p->wsig;
       DOWNDAT *downp = specp->downsrcp;
       if (downp->lofrq > FL(5.0)) {
-        sprintf(strmsg,
-                Str("instr %d %s, dft (%s), %ld octaves (%d - %d Hz):"),
+        snprintf(strmsg, 256,
+                Str("instr %d %s, dft (%s), %d octaves (%d - %d Hz):"),
                 (int) p->h.insdshead->p1, p->h.optext->t.inlist->arg[0],
                 outstring[specp->dbout],
                 downp->nocts, (int)downp->lofrq, (int)downp->hifrq);
       }
       else {                            /* more detail if low frequency  */
-        sprintf(strmsg,
-                Str("instr %d %s, dft (%s), %ld octaves (%3.1f - %3.1f Hz):"),
+        snprintf(strmsg, 256,
+                Str("instr %d %s, dft (%s), %d octaves (%3.1f - %3.1f Hz):"),
                 (int) p->h.insdshead->p1, p->h.optext->t.inlist->arg[0],
                 outstring[specp->dbout],
                 downp->nocts, downp->lofrq, downp->hifrq);
