@@ -248,10 +248,10 @@ static CS_NOINLINE int csoundLoadExternal(CSOUND *csound,
     if (UNLIKELY(err)) {
       char ERRSTR[256];
 #if !(defined(NACL)) && defined(LINUX)
-      sprintf(ERRSTR, Str("could not open library '%s' (%s)"),
+      snprintf(ERRSTR, 256, Str("could not open library '%s' (%s)"),
                       libraryPath, dlerror());
  #else
-      sprintf(ERRSTR, Str("could not open library '%s' (%d)"),
+      snprintf(ERRSTR, 256, Str("could not open library '%s' (%d)"),
                       libraryPath, err);
  #endif
       if (csound->delayederrormessages == NULL) {
@@ -364,7 +364,7 @@ static int csoundCheckOpcodeDeny(const char *fname)
     /* printf("DEBUG %s(%d): check fname=%s\n", __FILE__, __LINE__, fname); */
     /* printf("DEBUG %s(%d): list %s\n", __FILE__, __LINE__, list); */
     if (list==NULL) return 0;
-    strcpy(buff, fname);
+    strncpy(buff, fname, 256);
     strrchr(buff, '.')[0] = '\0'; /* Remove .so etc */
     p = strdup(list);
     deny = cs_strtok_r(p, ",", &th);
@@ -422,12 +422,12 @@ int csoundLoadModules(CSOUND *csound)
     }
     dir = opendir(dname);
     if (UNLIKELY(dir == (DIR*) NULL)) {
-      if(dname != NULL)
+      //if (dname != NULL)  /* cannot be other */
       csound->Warning(csound, Str("Error opening plugin directory '%s': %s"),
                                dname, strerror(errno));
-      else
-       csound->Warning(csound, Str("Error opening plugin directory: %s"),
-                               strerror(errno));
+      //else
+      //csound->Warning(csound, Str("Error opening plugin directory: %s"),
+      //                         strerror(errno));
       return CSOUND_SUCCESS;
     }
     /* load database for deferred plugin loading */
@@ -470,7 +470,7 @@ int csoundLoadModules(CSOUND *csound)
         csoundWarning(csound, Str("Library %s omitted\n"), fname);
         continue;
       }
-      sprintf(buf, "%s%c%s", dname, DIRSEP, fname);
+      snprintf(buf, 1024, "%s%c%s", dname, DIRSEP, fname);
       if (csound->oparms->odebug) {
         csoundMessage(csound, Str("Loading '%s'\n"), buf);
       }
@@ -532,7 +532,7 @@ int csoundLoadExternals(CSOUND *csound)
     } while (++i < cnt);
     /* file list is no longer needed */
     free(lst);
-    mfree(csound, s);
+    csound->Free(csound, s);
     return 0;
 }
 
@@ -833,7 +833,7 @@ const INITFN staticmodules[] = { hrtfopcodes_localops_init, babo_localops_init,
                                  fareyseq_localops_init, hrtfearly_localops_init,
                                  hrtfreverb_localops_init, minmax_localops_init,
                                  vaops_localops_init, pvsgendy_localops_init,
-#ifndef WIN32
+#ifdef LINUX
                                  cpumeter_localops_init,
 #endif
 #ifndef NACL
