@@ -24,6 +24,11 @@
 */
 
 #include "csoundCore.h"
+#ifdef HAVE_STRTOD_L
+static locale_t csound_c_locale = NULL;
+#else
+static char *csound_c_locale = NULL;
+#endif
 
 #ifdef HAVE_DIRENT_H
 #  include <sys/types.h>
@@ -41,10 +46,18 @@ int closedir(DIR*);
 #define CSSTRNGS_VERSION 0x2000
 #include <locale.h>
 #ifndef GNU_GETTEXT
-void init_getstring(void)
+void init_getstring(void *cs)
 {
-    setlocale(LC_NUMERIC, "C"); /* Ensure C syntax */
+#ifndef HAVE_STRTOD_L
+    setlocale(LC_NUMERIC, "C");                   /* Ensure C syntax */
+    csound_c_locale = setlocale(LC_NUMERIC, "C"); /* and remwmber */
+#else
+    if (csound_c_locale == NULL) {
+        csound_c_locale = newlocale (0, "C", NULL);
+    }
+#endif
 }
+
 PUBLIC char *csoundLocalizeString(const char *s)
 {
     return (char*)s;
@@ -55,25 +68,31 @@ PUBLIC void csoundSetLanguage(cslanguage_t lang_code)
     return;
 }
 #else
-void init_getstring(void)
+void init_getstring(void *cs)
 {
 /*     s = csoundGetEnv(NULL, "CS_LANG"); */
 /*     if (s == NULL)              /\* Default locale *\/ */
 /*       setlocale (LC_MESSAGES, ""); */
 /*     else  */
 /*       setlocale (LC_MESSAGES, s);    /\* Set to particular value *\/ */
-/*    textdomain("csound5"); */  /* This is not needed when using dgettext */
-    /* bind_textdomain_codeset("csound5", "UTF-8"); */
+/*    textdomain("csound6"); */  /* This is not needed when using dgettext */
+    /* bind_textdomain_codeset("csound6", "UTF-8"); */
 #ifdef never
     /* This is experimental; where should these be?? */
-    bindtextdomain("csound5", "/home/jpff/Sourceforge/csound/csound5/po");
+    bindtextdomain("csound6", "/home/jpff/Sourceforge/csound/csound6/po");
 #endif
+#ifndef HAVE_STRTOD_L
     setlocale(LC_NUMERIC, "C"); /* Ensure C syntax */
+#else
+    if (csound_c_locale == NULL) {
+        csound_c_locale = newlocale (0, "C", NULL);
+    }
+#endif
 }
 
 PUBLIC char *csoundLocalizeString(const char *s)
 {
-    return dgettext("csound5", s);
+    return dgettext("csound6", s);
 }
 
 static const char *language_names[] = {"", /* Default */
@@ -164,5 +183,123 @@ PUBLIC void csoundSetLanguage(cslanguage_t lang_code)
     }
     return;
 }
+
 #endif
 
+PUBLIC char* cs_strtok_r(char* str, char* delim, char** nextp) {
+#ifdef HAVE_STRTOK_R
+    return strtok_r(str, delim, nextp);
+#else
+    /*
+     * public domain strtok_r() by Charlie Gordon
+     *
+     *   from comp.lang.c  9/14/2007
+     *
+     *      http://groups.google.com/group/comp.lang.c/msg/2ab1ecbb86646684
+     *
+     *     (Declaration that it's public domain):
+     *      http://groups.google.com/group/comp.lang.c/msg/7c7b39328fefab9c
+     */
+    char *ret;
+    if (str == NULL)
+    {
+        str = *nextp;
+    }
+    str += strspn(str, delim);
+    if (*str == '\0')
+    {
+        return NULL;
+    }
+    ret = str;
+    str += strcspn(str, delim);
+    if (*str)
+    {
+        *str++ = '\0';
+    }
+    *nextp = str;
+    return ret;
+#endif
+}
+
+PUBLIC double cs_strtod(char* nptr, char** endptr) {
+#ifdef HAVE_STRTOD_L
+    return strtod_l(nptr, endptr, csound_c_locale);
+#else
+    return strtod(nptr, endptr);
+#endif
+}
+
+#if defined(HAVE_SPRINTF_L)
+PUBLIC int cs_sprintf(char *str, const char *format, ...)
+{
+    // This is not thread-safe but no idea how to fix
+    va_list args;
+    int retVal;
+    va_start(args, format);
+    retVal = vsprintf_l(str,csound_c_locale,format,args);
+    va_end(args);
+    return retVal;
+}
+
+PUBLIC int cs_sscanf(char *str, const char *format, ...)
+{
+    // This is not thread-safe but no idea how to fix
+    va_list args;
+    int retVal;
+    va_start(args, format);
+    retVal = vsscanf_l(str,csound_c_locale,format,args);
+    va_end(args);
+    return retVal;
+}
+#else
+#if defined(HAVE__SPRINT_L)
+PUBLIC int cs_sprintf(char *str, const char *format, ...)
+{
+    // This is not thread-safe but no idea how to fix
+    va_list args;
+    int retVal;
+    va_start(args, format);
+    retVal = __vsprintf_l(str,csound_c_locale,format,args);
+    va_end(args);
+    return retVal;
+}
+
+PUBLIC int cs_sscanf(char *str, const char *format, ...)
+{
+    // This is not thread-safe but no idea how to fix
+    va_list args;
+    int retVal;
+    va_start(args, format);
+    retVal = __vsscanf_l(str,csound_c_locale,format,args);
+    va_end(args);
+    return retVal;
+}
+#else
+PUBLIC int cs_sprintf(char *str, const char *format, ...)
+{
+    // This is not thread-safe but no idea how to fix
+    va_list args;
+    int retVal;
+    char *curlocale = setlocale(LC_NUMERIC, "C");
+    va_start(args, format);
+    retVal = vsprintf(str,format,args);
+    va_end(args);
+    setlocale(LC_NUMERIC, curlocale);
+    return retVal;
+}
+
+PUBLIC int cs_sscanf(char *str, const char *format, ...)
+{
+    // This is not thread-safe but no idea how to fix
+    va_list args;
+    int retVal;
+    char *curlocale = setlocale(LC_NUMERIC, "C");
+    va_start(args, format);
+    retVal = vsscanf(str,format,args);
+    va_end(args);
+    setlocale(LC_NUMERIC, curlocale);
+    return retVal;
+}
+
+#endif
+#endif
