@@ -68,6 +68,7 @@ struct CsoundPlugin {
 
   CsoundPlugin(const char *csd, int chns, int ports, AuxData* paux,
                unsigned long rate);
+  ~CsoundPlugin();
   void Process(unsigned long cnt);
 };
 
@@ -114,10 +115,19 @@ CsoundPlugin::CsoundPlugin(const char *csd,
   result = csound->Compile(5,cmdl);
   spout = csound->GetSpout();
   spin  = csound->GetSpin();
+  memset(ctl, 0, sizeof(LADSPA_Data *)*MAXPORTS);
 
   delete[] cmdl;
   delete[]  sr;
   delete[]  kr;
+  delete paux;
+}
+
+CsoundPlugin::~CsoundPlugin(){
+  delete csound;
+  delete[] ctlchn;
+  delete[] inp;
+  delete[] outp;
 }
 
 
@@ -177,9 +187,6 @@ static LADSPA_Handle createplugin(const LADSPA_Descriptor *pdesc,
 static void destroyplugin(LADSPA_Handle inst)
 {
   CsoundPlugin* p = (CsoundPlugin*)inst;
-  delete p->csound;
-  delete[] p->inp;
-  delete[] p->outp;
   delete p;
 }
 
@@ -388,13 +395,16 @@ static LADSPA_Descriptor *init_descriptor(char *csdname)
         desc->ImplementationData = (void *) paux;
         delete[] str;
         cerr << "PLUGIN LOADED\n";
-
         return desc;
       }
+      delete[] str;
     }
   // otherwise we just delete the empty descriptors
   // and return NULL
   delete desc;
+  delete[] PortNames;
+  delete paux;
+  delete[] ctlchn;
   delete[] PortDescriptors;
   delete[] PortRangeHints;
   cerr << "PLUGIN NOT LOADED: probably missing csLADSPA section\n";
@@ -436,7 +446,7 @@ unsigned int CountCSD(char **csdnames)
 #endif
     if(indx!=string::npos){
       dip = opendir(path.substr(0,indx).c_str());
-      strcpy(ladspa_path, path.substr(0,indx).c_str());
+      strncpy(ladspa_path, path.substr(0,indx).c_str(), 1024);
     }
     else dip = opendir(ladspa_path);
   }
@@ -450,7 +460,7 @@ unsigned int CountCSD(char **csdnames)
       string validExt = trim(temp.substr(indx+1));
       if(!validExt.compare("csd"))
         {
-          if(ladspa_path != NULL) {
+          if(strlen(ladspa_path) != 0) {
             name = ladspa_path;
             name.append("/");
             name.append(temp);
@@ -461,6 +471,7 @@ unsigned int CountCSD(char **csdnames)
           i++;
         }
     }
+  closedir(dip);
   return i;
 }
 
@@ -485,6 +496,7 @@ const LADSPA_Descriptor *ladspa_descriptor(unsigned long Index)
   // delete the CSD list
   for(unsigned int i=0; i < csds; i++) delete[] csdnames[i];
 
+  delete[] csdnames;
   if(descriptor == NULL)
     cerr << "no more csLADSPA plugins\n";
   return descriptor;
