@@ -149,6 +149,7 @@ static int scale(CSOUND *csound, int argc, char **argv)
     sc.end_table = &sc.scale_table;
 
     O.filetyp = O.outformat = 0;
+    O.heartbeat = 0;
     /* Check arguments */
     if ((envoutyp = csound->GetEnv(csound, "SFOUTYP")) != NULL) {
       if (strcmp(envoutyp, "AIFF") == 0)
@@ -238,7 +239,7 @@ static int scale(CSOUND *csound, int argc, char **argv)
           default:
             {
               char  err_msg[64];
-              sprintf(err_msg, Str("unknown flag -%c"), c);
+              snprintf(err_msg, 64, Str("unknown flag -%c"), c);
               usage(csound, err_msg);
             }
           }
@@ -250,6 +251,7 @@ static int scale(CSOUND *csound, int argc, char **argv)
 
  retry:
     /* Read sound file */
+    if (inputfile == NULL) return -1;
     if (!(infile = SCsndgetset(csound, &sc, inputfile))) {
       csound->Message(csound, Str("%s: error while opening %s"),
                               argv[0], inputfile);
