@@ -385,7 +385,7 @@ int main(int argc, char **argv)
     while (--j) {
       i++;
       if (argv[i][0] != '-' && i < 3 && tmp[0] == '\0') {
-        strcpy(tmp, argv[i]);
+        strncpy(tmp, argv[i],256);
         /* strip extension */
         if (is_orc(tmp) || is_sco(tmp) || is_mid(tmp) || is_csd(tmp))
           tmp[strlen(tmp) - 4] = '\0';
@@ -428,19 +428,19 @@ int main(int argc, char **argv)
     /*   (the last option has the highest precedence) */
     s = getenv("CSOUND");
     if (s != NULL)      /* get default setting from CSOUND, if available */
-      strcpy(tmp, s);
+      strncpy(tmp, s, 256);
     for (i = (int) strlen(optlst); --i >= 0; ) {
       sprintf(tmp2, "CSOUND_%c", optlst[i]);
       s = getenv(tmp2);
       if (s != NULL) {
-        strcpy(tmp, s);
+        strncpy(tmp, s, 256);
         if (tmp[0] != '\0') break;
       }
     }
     /*   if nothing was found, use default settings */
     if (tmp[0] == '\0') {
-      if      (opts[17]) strcpy(tmp, default_csnd_r);
-      else               strcpy(tmp, default_csnd);
+      if      (opts[17]) strncpy(tmp, default_csnd_r, 256);
+      else               strncpy(tmp, default_csnd, 256);
     }
     /* convert this to a list of quoted options */
     s2 = cmdline;
